import 'package:easyflu/easyflu.dart';
import 'package:flutter/cupertino.dart';
import 'package:easyflu/base/request_info.dart';
import 'package:easyflu/config/const.dart';
import 'package:easyflu/http/exception/http_error.dart';
import 'package:easyflu/http/http_data.dart';
import 'package:easyflu/utils/object_util.dart';
import 'base_view_state.dart';

enum LoadType {
  Refresh,
  LoadMore,
}

mixin BaseRefreshListController<T> on BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  List<T> _dataList = [];

  /// 当前页码
  int _currentPageNum = 0;

  int get currentPageNum => _currentPageNum;

  bool get refreshClearData => true;

  int? totalNum;

  void beforeRefresh(){

  }

  void beforeLoadMore(){

  }

  ValueNotifier<bool> _hasMoreValueNotifier = ValueNotifier(true);

  set hasMore(bool hasMore) => _hasMoreValueNotifier.value = hasMore;

  bool get hasMore => _hasMoreValueNotifier.value;


  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------


  void setList(LoadType loadType, HttpData httpData){
    list.addAll(httpData.data);
  }

  Future<HttpData> getData(LoadType loadType) async{

    bool _hasMore = false;

    HttpData httpData = await requestList<List<T>, T>((httpData, requestInfo) {

      if(requestInfo != null){
        _hasMore = requestInfo.useLoadMore;
      }

      if(currentPageNum == firstPage() && refreshClearData){
        list.clear();
      }

      setList(loadType, httpData);

      onCompleted(list);
    }, isSetError: currentPageNum == firstPage());


    _dataList.clear();

    if(httpData.data != null){
      _dataList.addAll(httpData.data);
    }else{
      _dataList.clear();
    }

    if(httpData.success && ObjectUtil.isNotEmpty(_dataList)){
      setIdle();
    }



    bool success = true;

    if(ObjectUtil.isNotEmpty(httpData.httpError)){
      if(loadType == LoadType.LoadMore){
        _currentPageNum = --_currentPageNum;
      }

      if(httpData.httpError != null){
        if(httpData.httpError!.httpErrorType != HttpErrorType.EmptyDataError){
          success = false;
        }
      }
    }

    refreshController.finishRefresh(success: success);
    refreshController.finishLoad(success: success);



    if(totalNum != null){
      _hasMore = currentPageNum < totalNum!;
    }else{
      _hasMore = ObjectUtil.isNotEmpty(httpData.data);
    }
    _hasMoreValueNotifier.value = _hasMore;
    return httpData;
  }




  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      getData(LoadType.Refresh);
    }
  }


  onCompleted(List<T> data) {}


  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  Future<HttpData> refreshData() {
    _currentPageNum = firstPage();
    beforeRefresh();
    return getData(LoadType.Refresh);
  }


  /// 上拉加载更多
  Future<HttpData> loadMore() {
    _currentPageNum = ++_currentPageNum;
    beforeLoadMore();
    return getData(LoadType.LoadMore);
  }




  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  Widget buildEasyRefreshWidget({
    ScrollController? scrollController,
    Widget? footWidget,
    ViewStateWidgetBuilder? emptyWidgetBuilder,
    ViewStateWidgetBuilder? errorWidgetBuilder,
    ViewStateModelWidgetListBuilder<List<T>>? widgetListBuilder,
    ViewStateModelWidgetBuilder<List<T>>? widgetBuilder,
    Widget? busyWidget,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function? onLoad,
    Function? onRefresh,
    }){
      return ValueListenableBuilder(
        valueListenable: _hasMoreValueNotifier,
        builder: (BuildContext context, bool value, _){
          return buildViewStateEasyRefresh<List<T>>(
              list,
              scrollController: scrollController,
              footWidget: footWidget,
              errorWidgetBuilder: errorWidgetBuilder,
              emptyWidgetBuilder: emptyWidgetBuilder,
              widgetListBuilder: widgetListBuilder,
              widgetBuilder: widgetBuilder,
              busyWidget: busyWidget,
              showBusyWidget: showBusyWidget,
              showEmptyWidget: showEmptyWidget,
              showErrorWidget: showErrorWidget,
              onLoad: value? onLoad??(){
                loadMore();
              } : null,
              onRefresh: onRefresh??(){
                refreshData();
              }
          );
        },
      );
  }




}
